#ifndef SCENETREE_H
#define SCENETREE_H

#include <vector>
#include <string>
#include "scene.h"
#include "light.h"
#include "resourcemanager.h"
#include <trackballcamera.h>

class SimplePipeline;
class SceneTree;

/**
 * @brief The SceneNode class represents a class of the game
 * that will be updated or used for rendering every frame
 */
class SceneNode
{
public:
    SceneNode* m_parent;
    virtual void update() = 0;
    virtual Light* getLight() {return nullptr;}
    virtual GeometryNode* getGeometryNode() {return nullptr;}
    virtual void addedToSceneTree(SceneTree* tree){}
};

/**
 * @brief The ContainerNode class represents a node which main use is to contain a collection of nodes
 */
class ContainerNode : public SceneNode
{
public:
    virtual void update();

    /**
     * @brief addChild adds node in the ContainerNode's children list
     */
    void addChild(SceneNode* node);

    /**
     * @brief removeChild removes the first iteration of node of the children list
     */
    void removeChild(SceneNode* node);
    /**
     * @brief addedToSceneTree is called when this node is added to a SceneTree
     *
     * @param tree
     */
    void addedToSceneTree(SceneTree *tree);

protected:
    std::vector<SceneNode*> m_children;
};

/**
 * @brief The MeshNode class holds a mesh
 */
class MeshNode : public SceneNode
{
    GeometryNode geometry;
public:
    MeshNode(Mesh* mesh) : geometry(mesh, glm::mat4()) {}

    virtual void update() {}

    void setTransform(const glm::mat4 &transform) { geometry.modelMatrix = transform; }
    const glm::mat4& getTransform() { return geometry.modelMatrix; }

    GeometryNode* getGeometryNode() { return &geometry; }
};

/**
 * @brief The CameraNode class is a scene node that can be used by the renderer
 */
class CameraNode : public TrackBallCamera, SceneNode
{
public:
    CameraNode() {}
};

class SceneTree : public Scene
{
public:
    SceneTree();
    ~SceneTree();

    virtual SceneIterator<Light*>* getLights();
    virtual SceneIterator<GeometryNode*>* getGeometry();

    void update();

    Texture* getSkybox() {return m_skybox;}
    void setSkybox(Texture* skybox) {m_skybox = skybox;}

    void setMainCamera(CameraNode *camera);

    ContainerNode* getRootObject(){return &m_root;}
    void addObject(ContainerNode *parent, SceneNode *node);
    void addToIndex(SceneNode* node);
    void removeObject(ContainerNode* parent,SceneNode *node);
    void removeFromIndex(SceneNode *node);
private:
    ContainerNode m_root;
    std::vector<Light*> m_lights;
    std::vector<GeometryNode*> m_geometries;
    Texture* m_skybox;
};

// Additionnal Nodes :

#endif // SCENETREE_H
